@@ -84,6 +84,9 @@ sub vhost_selector {
 
     my $vhost = $req->header("Host");
 
+    # Strip off the :portnumber, if any
+    $vhost =~ s/:\d+$//;
+
     my $maps = $cb->{service}{extra_config}{_vhosts} ||= {};
 
     # returns 1 if done with client, 0 if no action taken
